hpgsql
Safe HaskellNone
LanguageHaskell2010

Hpgsql.Notification

Synopsis

Documentation

getNotification :: HPgConnection -> IO NotificationResponse Source #

Blocks until a new Notification arrives. This function is both thread-safe and interruption-safe, so you can run it with the same connection in parallel to any other functions.

getNotificationNonBlocking :: HPgConnection -> IO (Maybe NotificationResponse) Source #

This function is thread-and-interruption-safe, so you can run it with the same connection in parallel to any other functions.