| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
Hpgsql.Notification
Synopsis
Documentation
getNotification :: HPgConnection -> IO NotificationResponse Source #
Blocks until a new Notification arrives. This function is both thread-safe and interruption-safe, so you can run it with the same connection in parallel to any other functions.
getNotificationNonBlocking :: HPgConnection -> IO (Maybe NotificationResponse) Source #
This function is thread-and-interruption-safe, so you can run it with the same connection in parallel to any other functions.
data NotificationResponse Source #
Constructors
| NotificationResponse | |
Fields
| |
Instances
| Show NotificationResponse Source # | |
Defined in Hpgsql.InternalTypes Methods showsPrec :: Int -> NotificationResponse -> ShowS # show :: NotificationResponse -> String # showList :: [NotificationResponse] -> ShowS # | |
| Eq NotificationResponse Source # | |
Defined in Hpgsql.InternalTypes Methods (==) :: NotificationResponse -> NotificationResponse -> Bool # (/=) :: NotificationResponse -> NotificationResponse -> Bool # | |